#include <QFileDialog>
#include <QString>
#include <QTimer>
#include <QProcess>
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "downloader.h"



MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->progressBar->setRange(0,100);
    ui->progressBar->setValue(0);

    timer = new QTimer(this);

    ui->statusbar->setText("Valmiina lataamaan!");
    for(int i=0;i<5;i++)
    {
        ui->threadcount->addItem(QString::number(i+1));
        threadActive[i]=0;
    }
    ui->searchTarget->addItem("video");
    ui->searchTarget->addItem("audio");
    completed=0;
    totalfilecount=0;
}

MainWindow::~MainWindow()
{
    delete ui;
    delete timer;    
}

void MainWindow::DialogOpen()
{
    static QFileDialog load(this, "Tallenna kansioon..", ".",0);
    load.setFileMode(QFileDialog::Directory);
    load.setConfirmOverwrite(true);
    load.show();    
    connect(&load, SIGNAL(fileSelected(QString)), this, SLOT(fileSelected(QString)));
}

void MainWindow::fileSelected(const QString &file)
{
    ui->outputfile->setText(file);
}

void MainWindow::StartProcessing()
{
    StartProcessing2(0);
}

void MainWindow::StartProcessing2(int onlyone=0)
{
    ui->pushButton->setDisabled(true);
    QString download;

    if(!totalfilecount)
    {
        totalfilecount=ui->url->text().count(";")+1;
        ui->statusbar->setText("Ladataan tiedostoja.."+ui->threadcount->currentText()+"/"+QString::number(totalfilecount));
    }
    for(int i=0;i<(onlyone?1:ui->threadcount->currentText().toInt());i++)
    {

        //If multiple downloads
        if(ui->url->text().indexOf(";")!=-1)
        {            
            multiple=1;
            download=ui->url->text().mid(0, ui->url->text().indexOf(";"));
            ui->url->setText(ui->url->text().mid(ui->url->text().indexOf(";")+1));            
        }
        else
        {
            download=ui->url->text();
            multiple=0;            
        }

        for(int i=0;i<5;i++)
        {
            if(!threadActive[i])
            {
                downloads[i]=new downloader(i, ((ui->checkBox->checkState()==Qt::Checked)?1:0), download, ui->outputfile->text());
                threadActive[i]=1;
                connect(downloads[i], SIGNAL(valueChanged(int, int, int, int, QString)), this, SLOT(valueChanged(int, int, int, int, QString)));
                break;
            }
        }
        if(!multiple) break;

    }
    ui->outputfile->setDisabled(true);
}

void MainWindow::calculateProgress()
{
    int progPerOne=(int)(100.0f/totalfilecount);
    int completeProg=progPerOne*completed;
    for(int i=0;i<5;i++)
    {
        if(threadActive[i])
        {
            completeProg+=(int)((progress[i]/100.0f)*progPerOne);
        }
    }
    ui->progressBar->setValue(completeProg);
}

void MainWindow::valueChanged(int ID, int percent1, int percent2, int err, QString errmsg)
{
    //Print out error message if there was an error
    if(err)
    {
        ui->statusbar->setText(errmsg);
        delete downloads[ID];
        threadActive[ID]=0;
        completed++;

        calculateProgress();
        //If more to download..
        if(multiple) StartProcessing2(1);
    }
    else
    {
        if(percent1 == 100 && percent2 == 100)
        {
            delete downloads[ID];
            threadActive[ID]=0;
            completed++;
            progress[ID]=0;
            ui->statusbar->setText("Valmiina "+QString::number(completed)+"/"+QString::number(totalfilecount));
            //If more to download..
            if(multiple) StartProcessing2(1);
        }
        else
        {
             progress[ID]=percent1;
        }
    }

    if(completed==totalfilecount)
    {
        ui->pushButton->setDisabled(false);
        ui->outputfile->setDisabled(false);
        ui->statusbar->setText("Tiedosto(t) onnistuneesti tallennettu!");
        completed=0;
        totalfilecount=0;
        ui->progressBar->setValue(100);
        timer->stop();
        for(int i=0;i<5;i++)
            progress[i]=0;
    }
    if(totalfilecount) calculateProgress();
}


void MainWindow::startSearch()
{    
    search = new searchWindow(this,ui->searchword->text(), ui->url, ui->searchTarget->currentText());
    search->show();
    search->doSearch();
}
